/* include/config.h.in.  Generated from configure.in by autoheader.  */
/*
dnl $Id: config.h,v 1.4 2004/01/03 20:31:00 mike Exp $
dnl
dnl Libnet autoconfiguration acconfig.h file
dnl Copyright (c) 1998 - 2004 Mike D. Schiffman <mike@infonexus.com>
dnl All rights reserved.
dnl
dnl Process this file with autoheader to produce a config.h file.
dnl
*/

#undef LIBNET_BSDISH_OS
#undef LIBNET_BSD_BYTE_SWAP
#undef DLPI_DEV_PREFIX
#undef HAVE_DEV_DLPI
#undef HAVE_SOLARIS
#undef HAVE_SOLARIS_IPV6
#undef HAVE_HPUX11
#undef HAVE_SOCKADDR_SA_LEN
#undef HAVE_DLPI
#undef HAVE_PACKET_SOCKET
#undef HAVE_STRUCT_IP_CSUM
#undef HAVE_LIB_PCAP
#undef LBL_ALIGN
#undef STUPID_SOLARIS_CHECKSUM_BUG
#undef _BSD_SOURCE
#undef __BSD_SOURCE
#undef __FAVOR_BSD
#undef LIBNET_BIG_ENDIAN
#define LIBNET_LIL_ENDIAN 1
#undef NO_SNPRINTF


/*
dnl EOF
*/

/* Define to 1 if you have the <inttypes.h> header file. */
#undef HAVE_INTTYPES_H

/* Define to 1 if you have the `nsl' library (-lnsl). */
#undef HAVE_LIBNSL

/* Define to 1 if you have the `packet' library (-lpacket). */
#undef HAVE_LIBPACKET

/* Define to 1 if you have the `socket' library (-lsocket). */
#undef HAVE_LIBSOCKET

/* Define to 1 if you have the `wpcap' library (-lwpcap). */
#undef HAVE_LIBWPCAP

/* Define to 1 if you have the <memory.h> header file. */
#undef HAVE_MEMORY_H

/* Define to 1 if you have the <net/ethernet.h> header file. */
#define HAVE_NET_ETHERNET_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#undef HAVE_STDINT_H

/* Define to 1 if you have the <stdlib.h> header file. */
#undef HAVE_STDLIB_H

/* Define to 1 if you have the `strerror' function. */
#define HAVE_STRERROR 1

/* Define to 1 if you have the <strings.h> header file. */
#undef HAVE_STRINGS_H

/* Define to 1 if you have the <string.h> header file. */
#undef HAVE_STRING_H

/* Define to 1 if you have the <sys/bufmod.h> header file. */
#undef HAVE_SYS_BUFMOD_H

/* Define to 1 if you have the <sys/dlpi_ext.h> header file. */
#undef HAVE_SYS_DLPI_EXT_H

/* Define to 1 if you have the <sys/sockio.h> header file. */
#undef HAVE_SYS_SOCKIO_H

/* Define to 1 if you have the <sys/stat.h> header file. */
#undef HAVE_SYS_STAT_H

/* Define to 1 if you have the <sys/types.h> header file. */
#undef HAVE_SYS_TYPES_H

/* Define to 1 if you have the <unistd.h> header file. */
#undef HAVE_UNISTD_H

/* Name of package */
#undef PACKAGE

/* Define to the address where bug reports for this package should be sent. */
#undef PACKAGE_BUGREPORT

/* Define to the full name of this package. */
#undef PACKAGE_NAME

/* Define to the full name and version of this package. */
#undef PACKAGE_STRING

/* Define to the one symbol short name of this package. */
#undef PACKAGE_TARNAME

/* Define to the version of this package. */
#undef PACKAGE_VERSION

/* Define to 1 if you have the ANSI C header files. */
#undef STDC_HEADERS
