/*
dnl $Id: acconfig.h,v 1.2 2004/01/03 20:31:00 mike Exp $
dnl
dnl Libnet autoconfiguration acconfig.h file
dnl Copyright (c) 1998 - 2004 Mike D. Schiffman <mike@infonexus.com>
dnl All rights reserved.
dnl
dnl Process this file with autoheader to produce a config.h file.
dnl
*/

#undef LIBNET_BSDISH_OS
#undef LIBNET_BSD_BYTE_SWAP
#undef DLPI_DEV_PREFIX
#undef HAVE_DEV_DLPI
#undef HAVE_SOLARIS
#undef HAVE_SOLARIS_IPV6
#undef HAVE_HPUX11
#undef HAVE_SOCKADDR_SA_LEN
#undef HAVE_DLPI
#undef HAVE_PACKET_SOCKET
#undef HAVE_STRUCT_IP_CSUM
#undef HAVE_LIB_PCAP
#undef LBL_ALIGN
#undef STUPID_SOLARIS_CHECKSUM_BUG
#undef _BSD_SOURCE
#undef __BSD_SOURCE
#undef __FAVOR_BSD
#undef LIBNET_BIG_ENDIAN
#undef LIBNET_LIL_ENDIAN
#undef NO_SNPRINTF


/*
dnl EOF
*/
